@@ -5,9 +5,15 @@ from __future__ import absolute_import
 import json
 import os
 
+try:
+    from unittest.mock import MagicMock
+except ImportError:
+    from mock import MagicMock
+
 import pytest
 
 import click
+import kaptan
 from click.testing import CliRunner
 
 import libtmux
@@ -15,13 +21,18 @@ from libtmux.common import has_lt_version
 from libtmux.exc import LibTmuxException
 from tmuxp import cli, config, exc
 from tmuxp.cli import (
+    _load_append_windows_to_current_session,
+    _load_attached,
+    _reattach,
     command_debug_info,
     command_ls,
     get_config_dir,
     is_pure_name,
+    load_plugins,
     load_workspace,
     scan_config,
 )
+from tmuxp.workspacebuilder import WorkspaceBuilder
 
 from .fixtures._util import curjoin, loadfixture
 
@@ -983,6 +994,215 @@ def test_ls_cli(monkeypatch, tmpdir):
     assert cli_output == '\n'.join(stems) + '\n'
 
 
+def test_load_plugins(monkeypatch_plugin_test_packages):
+    from tmuxp_test_plugin_bwb.plugin import PluginBeforeWorkspaceBuilder
+
+    plugins_config = loadfixture("workspacebuilder/plugin_bwb.yaml")
+
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(plugins_config).get()
+    sconfig = config.expand(sconfig)
+
+    plugins = load_plugins(sconfig)
+
+    assert len(plugins) == 1
+
+    test_plugin_class_types = [
+        PluginBeforeWorkspaceBuilder().__class__,
+    ]
+    for plugin in plugins:
+        assert plugin.__class__ in test_plugin_class_types
+
+
+@pytest.mark.skip('Not sure how to clean up the tmux session this makes')
+@pytest.mark.parametrize(
+    "cli_args,inputs",
+    [
+        (
+            ['load', 'tests/fixtures/workspacebuilder/plugin_versions_fail.yaml'],
+            ['y\n'],
+        )
+    ],
+)
+def test_load_plugins_version_fail_skip(
+    monkeypatch_plugin_test_packages, cli_args, inputs
+):
+    runner = CliRunner()
+
+    results = runner.invoke(cli.cli, cli_args, input=''.join(inputs))
+    assert '[Loading]' in results.output
+
+
+@pytest.mark.parametrize(
+    "cli_args,inputs",
+    [
+        (
+            ['load', 'tests/fixtures/workspacebuilder/plugin_versions_fail.yaml'],
+            ['n\n'],
+        )
+    ],
+)
+def test_load_plugins_version_fail_no_skip(
+    monkeypatch_plugin_test_packages, cli_args, inputs
+):
+    runner = CliRunner()
+
+    results = runner.invoke(cli.cli, cli_args, input=''.join(inputs))
+    assert '[Not Skipping]' in results.output
+
+
+@pytest.mark.parametrize(
+    "cli_args", [(['load', 'tests/fixtures/workspacebuilder/plugin_missing_fail.yaml'])]
+)
+def test_load_plugins_plugin_missing(monkeypatch_plugin_test_packages, cli_args):
+    runner = CliRunner()
+
+    results = runner.invoke(cli.cli, cli_args)
+    assert '[Plugin Error]' in results.output
+
+
+def test_plugin_system_before_script(
+    monkeypatch_plugin_test_packages, server, monkeypatch
+):
+    # this is an implementation test. Since this testsuite may be ran within
+    # a tmux session by the developer himself, delete the TMUX variable
+    # temporarily.
+    monkeypatch.delenv('TMUX', raising=False)
+    session_file = curjoin("workspacebuilder/plugin_bs.yaml")
+
+    # open it detached
+    session = load_workspace(
+        session_file, socket_name=server.socket_name, detached=True
+    )
+
+    assert isinstance(session, libtmux.Session)
+    assert session.name == 'plugin_test_bs'
+
+
+def test_reattach_plugins(monkeypatch_plugin_test_packages, server):
+    config_plugins = loadfixture("workspacebuilder/plugin_r.yaml")
+
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(config_plugins).get()
+    sconfig = config.expand(sconfig)
+
+    # open it detached
+    builder = WorkspaceBuilder(
+        sconf=sconfig, plugins=load_plugins(sconfig), server=server
+    )
+    builder.build()
+
+    try:
+        _reattach(builder)
+    except libtmux.exc.LibTmuxException:
+        pass
+
+    proc = builder.session.cmd('display-message', '-p', "'#S'")
+
+    assert proc.stdout[0] == "'plugin_test_r'"
+
+
+def test_load_attached(server, monkeypatch):
+    # Load a session and attach from outside tmux
+    monkeypatch.delenv('TMUX', raising=False)
+
+    attach_session_mock = MagicMock()
+    attach_session_mock.return_value.stderr = None
+
+    monkeypatch.setattr("libtmux.session.Session.attach_session", attach_session_mock)
+
+    yaml_config = loadfixture("workspacebuilder/two_pane.yaml")
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(yaml_config).get()
+
+    builder = WorkspaceBuilder(sconf=sconfig, server=server)
+
+    _load_attached(builder, False)
+
+    assert builder.session.attach_session.call_count == 1
+
+
+def test_load_attached_detached(server, monkeypatch):
+    # Load a session but don't attach
+    monkeypatch.delenv('TMUX', raising=False)
+
+    attach_session_mock = MagicMock()
+    attach_session_mock.return_value.stderr = None
+
+    monkeypatch.setattr("libtmux.session.Session.attach_session", attach_session_mock)
+
+    yaml_config = loadfixture("workspacebuilder/two_pane.yaml")
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(yaml_config).get()
+
+    builder = WorkspaceBuilder(sconf=sconfig, server=server)
+
+    _load_attached(builder, True)
+
+    assert builder.session.attach_session.call_count == 0
+
+
+def test_load_attached_within_tmux(server, monkeypatch):
+    # Load a session and attach from within tmux
+    monkeypatch.setenv('TMUX', "/tmp/tmux-1234/default,123,0")
+
+    switch_client_mock = MagicMock()
+    switch_client_mock.return_value.stderr = None
+
+    monkeypatch.setattr("libtmux.session.Session.switch_client", switch_client_mock)
+
+    yaml_config = loadfixture("workspacebuilder/two_pane.yaml")
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(yaml_config).get()
+
+    builder = WorkspaceBuilder(sconf=sconfig, server=server)
+
+    _load_attached(builder, False)
+
+    assert builder.session.switch_client.call_count == 1
+
+
+def test_load_attached_within_tmux_detached(server, monkeypatch):
+    # Load a session and attach from within tmux
+    monkeypatch.setenv('TMUX', "/tmp/tmux-1234/default,123,0")
+
+    switch_client_mock = MagicMock()
+    switch_client_mock.return_value.stderr = None
+
+    monkeypatch.setattr("libtmux.session.Session.switch_client", switch_client_mock)
+
+    yaml_config = loadfixture("workspacebuilder/two_pane.yaml")
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(yaml_config).get()
+
+    builder = WorkspaceBuilder(sconf=sconfig, server=server)
+
+    _load_attached(builder, True)
+
+    assert builder.session.switch_client.call_count == 1
+
+
+def test_load_append_windows_to_current_session(server, monkeypatch):
+    yaml_config = loadfixture("workspacebuilder/two_pane.yaml")
+    sconfig = kaptan.Kaptan(handler='yaml')
+    sconfig = sconfig.import_config(yaml_config).get()
+
+    builder = WorkspaceBuilder(sconf=sconfig, server=server)
+    builder.build()
+
+    assert len(server.list_sessions()) == 1
+    assert len(server._list_windows()) == 3
+
+    # Assign an active pane to the session
+    monkeypatch.setenv("TMUX_PANE", server._list_panes()[0]["pane_id"])
+
+    builder = WorkspaceBuilder(sconf=sconfig, server=server)
+    _load_append_windows_to_current_session(builder)
+
+    assert len(server.list_sessions()) == 1
+    assert len(server._list_windows()) == 6
+
+
 def test_debug_info_cli(monkeypatch, tmpdir):
     monkeypatch.setenv('SHELL', '/bin/bash')
 
